# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-08 18:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0002_auto_20160907_1912'),
    ]

    operations = [
        migrations.AddField(
            model_name='unshardedtestmodel',
            name='more_random_string',
            field=models.CharField(default=11, max_length=120),
            preserve_default=False,
        ),
    ]
