# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-08 18:38
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0004_remove_unshardedtestmodel_more_random_string'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='unshardedtestmodel',
            name='more_random_string',
        ),
    ]
